/*
Lexmark 1100 printer driver for Linux
Copyright (C) 1999   Tim Engler

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <string.h>
#include "DataImage.h"

DataImage::~DataImage()
{
	if(bytes)
		delete bytes;
}

DataImage::DataImage()
{
	bytes = 0;
}


void DataImage::cacheLines(int start, int len)
{
	int end;
	unsigned char *bytes;
	int offset;
	int array_offset;
	int rel_start;
	
	if(start < 0)
		start = 0;

	if(start+len > this->height)
		len = this->height-start;
	
	//	fprintf(stderr,"start %d end %d\n",start,start+len);

	start = start * this->byte_width;
	len = len * this->byte_width;

    end = start + len;

	if(start >= end) // data asked for outside of the image's range
		return;
	
	
    offset = this->end;
    array_offset=0;
    rel_start = start;

	bytes = new unsigned char [len];

    //if some of the lines are there already
    if(this->end > start)
    {
		if(end <= this->end)
		{
			delete bytes;
			return; //these lines are already cached
		}
	
		//copys old bytes to new array
		memcpy(bytes,&this->bytes[start - this->start],this->end - start);

		//only update bytes after length
		array_offset = this->end - start;
		len -= array_offset;
		rel_start = this->end;
    }

    //if the filepositon isn't in the right spot
    if(offset != rel_start)
    {
		this->reader->seek(rel_start-offset,1);
    }

    //read the data
    this->reader->read(&bytes[array_offset], len, 1);

	if(this->bytes) //if we were storing data
		delete [] this->bytes;
	
    this->bytes = bytes;

    this->start = start;
    this->end = end;
}

