/*
Lexmark 1100 printer driver for Linux
Copyright (C) 1999   Tim Engler

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "Map.h"
#include "errors.h"
#include <string.h>
#include <stdio.h>

Map::~Map()
{
	if(bytes)
		delete bytes;

	LinkedList *freeMe;

	while(rootAlg)
	{
		freeMe = rootAlg;
		rootAlg = rootAlg->next;
		delete freeMe;
	}
}

Map::Map(int width, int height,int cellSize)
{
	this->bytes = 0;
	this->start = 0;
	this->end = 0;	
	this->cellSize = cellSize;
	this->rootAlg = 0;

	this->width = width;
	this->height = height;
}

void Map::addAlgorithm(MapAlgorithm *algorithm)
{
	if(!rootAlg)
	{
		rootAlg = new LinkedList(algorithm);
	}
	else 
	{
		LinkedList * i;
		for(i = rootAlg; i->next != 0; i = i->next);
		
		i->next = new LinkedList(algorithm);
	}
}


void *Map::getCell(int x, int y)
{
	if(y < start || y >= end)
		die("y is out of range,(%d)\n",y);
	return &bytes[((y - start) * width + x)* cellSize];
}

void Map::cacheLines(int start, int len)
{
	int end;
	char *newBytes;
	int offset;
	int array_offset;
	
	if(start < 0)
	{
		len += start;
		start = 0;
	}
	

	if(start+len > this->height)
		len = this->height-start;
	
	if(len < 0) // data asked for outside of the image's range
		return;
	
	end = start+len;
	
    offset = this->end * this->width * this->cellSize;
    array_offset=0;

	newBytes = new char [len * this->width * this->cellSize];

    //if some of the lines are there already
    if(this->end > start)
    {
		if(end <= this->end)
		{
			delete newBytes;
			return; //these lines are already cached
		}
	
		//copys old bytes to new array
		memcpy(newBytes,&this->bytes[(start - this->start) * width * cellSize],(this->end - start)*width*cellSize);

		//only update bytes after length
		array_offset = (this->end - start) * width * cellSize;
		len -= this->end - start;
    }

	if(this->bytes) //if we were storing data
		delete [] this->bytes;
	
    this->bytes = newBytes;

    this->start = start;

    this->end = end;

	// run the algorithms on the new cells
	for(LinkedList * i = rootAlg; i != 0; i = i->next)
	{
		((MapAlgorithm *)i->object)->touchDataLines(this,array_offset / width / cellSize + start,len,&this->bytes[array_offset]);
	}

}

