/*
Lexmark 1100 printer driver for Linux
Copyright (C) 1999   Tim Engler

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "Reader.h"

Reader::Reader(FILE *fd, bool supportsSeek)
{
	this->fd = fd;
	this->supportsSeek = supportsSeek;
	this->filePos = 0;
}


int Reader::readLine(char *line, int max_len)
{
	int c;
	int pos = 0;

	//stdio should take care of buffering
	while((c=getc(this->fd)) != EOF && c != '\n' && pos < max_len-1)
		line[pos++] = c;

	line[pos] = '\0';

	if(pos == 0 && c == -1) //we didn't read a single character due to EOF
		return -1; // error

	filePos += pos;
	
	return pos;
}


int Reader::seek(int offset, int whence)
{
	if(supportsSeek)
	{
		int i = fseek(this->fd, offset, whence);
		fprintf(stderr,"fseek %d %d\n",i,offset);
		return i;
	}
	char buffer[255];
	
	if(whence == 2)
		return -1; // can't seek from end
	if(whence == 0)
		offset -= filePos;
	
	if(offset < 0)
		return -1; // can't seek backwards
	
	int c;
	while((c=fread(buffer,1,255 < offset ? 255 : offset,fd)) > 0)
		offset -= c;
	return 0;
}

size_t Reader::read(void *ptr, size_t size, size_t nmemb)
{
	int c = fread(ptr,size,nmemb,this->fd);
	if(c > 0)
		filePos+=c*size;
	return c;
}

long Reader::tell()
{
	if(supportsSeek)
	{
		int i = ftell(this->fd);
		fprintf(stderr,"tell %d\n",i);
		return i;
	}
	
	return filePos;
}

